# TWO FUNCTIONS
# calcZ(id="id", weeks="weeks", y=c("length", "weight", "hc"), sex="sex"=, SD23, data) calculates Z and percentile for weight, length, and hc using the Fenton growth charts 2013 (completed weeks version). Although you can enter decimal weeks 22-49, only completed weeks are considered i.e 24.6 weeks is treated as 24 completed weeks via floor(). Depending on whether y = "weight", "length", or "hc", an appropriate Z score will be returned.  SD23=TRUE requests the "WHO fudge" for extreme Z. GA in weeks, length in cm, and weight in grams. 

# makeZs(data) takes a dataset data= with columns id, weight, length, hc, sex, and weeks and returns the original dataset plus all 3 z-scores (SD=TRUE for weight). Missing anthropometric measures will return NA. At a minimum, you must have id, sex, and one anthropometric measure

makeZs<-function(data){
  # to avoid errors from calling calcZ() with missing columns
  if (is.null(data$length)) data$length<-rep(NA, nrow(data))
  if (is.null(data$weight)) data$weight<-rep(NA, nrow(data))
  if (is.null(data$hc)) data$hc<-rep(NA,nrow(data))
  
  tmp1<-calcZ(y="weight", SD23=TRUE, data=data)
  tmp11<-tmp1[,c("id","zscore","percentile")];names(tmp11)<-c("id","WZ","Wpct")
  tmp2<-calcZ(y="length", SD23=FALSE, data=data)
  tmp21<-tmp2[,c("id","zscore","percentile")];names(tmp21)<-c("id","LZ","Lpct")
  tmp3<-calcZ(y="hc", data=data, SD23=FALSE)
  tmp31<-tmp3[,c("id","zscore","percentile")];names(tmp31)<-c("id","HCZ","HCpct")
  tmp4<-merge(data, tmp11, by="id")
  tmp5<-merge(tmp4, tmp21, by="id")
  tmp6<-merge(tmp5, tmp31, by="id")
  return(tmp6)
}



####### FUNCTIONS

calcZ<-function(id="id", weeks="weeks", y, sex="sex", SD23=TRUE, data){
  
  data1<-data[,c(id, weeks, sex, y)]; 
  names(data1)<-c("id", "weeks","sex", "y")
  data1$weeks<-floor(data1$weeks)
  
  load("lms_girls_wt.Rdata")
  load("lms_girls_wt.Rdata"); 
  load("lms_girls_length.Rdata"); 
  load("lms_girls_hc.Rdata"); 
  load("lms_boys_wt.Rdata"); 
  load("lms_boys_length.Rdata"); 
  load("lms_boys_hc.Rdata"); 
  
  lmsM<-switch(y, "weight"=lms_boys_wt, "length"=lms_boys_length, "hc"=lms_boys_hc)
  lmsF<-switch(y, "weight"=lms_girls_wt, "length"=lms_girls_length, "hc"=lms_girls_hc)
  
  data1$zscore<-NULL # create place holders
  data1$percentile<-NULL # create place holders
  
  #### generic function as usual ########### 
  
  for(i in 1:dim(data1)[1]){
    obsi<-data1[i,];obsi
    if( is.na(obsi$y) | is.na(obsi$weeks) | is.na(obsi$sex)){data1$zscore[i]<-NA;data1$percentile[i]<-NA;next}
    
    if(obsi$sex %in% c("M","m",1)){lms<-lmsM}  
    if(obsi$sex %in% c("F","f",2)){lms<-lmsF} 
    lms_index<-which(lms$Compl.weeks == floor(obsi$weeks))[1];lms_index
    lms1<-lms[lms_index,];lms1
    l<-lms1$L;m<-lms1$M;s<-lms1$S; c(l,m,s)
    
    zind<-((data1$y[i]/m)^l-1)/(s*l);zind
    sd3pos<-m*(1+s*l*3)^(1/l);sd3pos
    sd3neg<-m*(1+s*l*(-3))^(1/l);sd3neg
    sd2pos<-m*(1+s*l*2)^(1/l);sd2pos
    sd2neg<-m*(1+s*l*(-2))^(1/l);sd2neg
    sd23pos<-sd3pos-sd2pos;sd23pos
    sd23neg<-sd2neg-sd3neg;sd23neg
    
    #check zind for NA, set zalpha, %ile accordintgly
    #check zind for in bounds and 'fudge' accordingly
    zalpha<-zind;zalpha # if -3 < z < 3
    if(is.na(zind)){
      zalpha<-NA;
      data1$zscore[i]<-NA;
      data1$percentile[i]<-NA
    } else {
      if(zind > 3 & SD23){zalpha<-3+(data1$y[i]-sd3pos)/sd23pos};zalpha
      if(zind < -3 & SD23){zalpha<--3+(data1$y[i]-sd3neg)/sd23neg};zalpha
      data1$zscore[i]<-round(zalpha,2);data1$zscore[i]
      percentile<-pnorm(zalpha);percentile
      data1$percentile[i]<-100*round(percentile,3);data1$percentile[i]
    } # end if-else fudge
  } #end i loop
  data1} # end wtz


############## use function  #########




